# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is auto-generated by scripts/generate_script_names.pl
# from the Unicode character database bundled with the running Perl.
# It maps Unicode script long names (as returned by
# Unicode::UCD::charscript()) to ISO 15924 four-letter codes.
#
# Regenerate with:
#   perl tools/generate_script_names.pl > lib/PersonName/Format/ScriptNames.pl

%PersonName::Format::PP::SCRIPT_NAME_TO_CODE = (
    'Adlam'                             => 'Adlm',
    'Ahom'                              => 'Ahom',
    'Anatolian_hieroglyphs'             => 'Hluw',
    'Arabic'                            => 'Arab',
    'Armenian'                          => 'Armn',
    'Avestan'                           => 'Avst',
    'Balinese'                          => 'Bali',
    'Bamum'                             => 'Bamu',
    'Bassa_vah'                         => 'Bass',
    'Batak'                             => 'Batk',
    'Bengali'                           => 'Beng',
    'Bhaiksuki'                         => 'Bhks',
    'Bopomofo'                          => 'Bopo',
    'Brahmi'                            => 'Brah',
    'Braille'                           => 'Brai',
    'Buginese'                          => 'Bugi',
    'Buhid'                             => 'Buhd',
    'Canadian_aboriginal'               => 'Cans',
    'Carian'                            => 'Cari',
    'Caucasian_albanian'                => 'Aghb',
    'Chakma'                            => 'Cakm',
    'Cham'                              => 'Cham',
    'Cherokee'                          => 'Cher',
    'Chorasmian'                        => 'Chrs',
    'Common'                            => 'Zyyy',
    'Coptic'                            => 'Copt',
    'Cuneiform'                         => 'Xsux',
    'Cypriot'                           => 'Cprt',
    'Cypro_minoan'                      => 'Cpmn',
    'Cyrillic'                          => 'Cyrl',
    'Deseret'                           => 'Dsrt',
    'Devanagari'                        => 'Deva',
    'Dives_akuru'                       => 'Diak',
    'Dogra'                             => 'Dogr',
    'Duployan'                          => 'Dupl',
    'Egyptian_hieroglyphs'              => 'Egyp',
    'Elbasan'                           => 'Elba',
    'Elymaic'                           => 'Elym',
    'Ethiopic'                          => 'Ethi',
    'Georgian'                          => 'Geor',
    'Glagolitic'                        => 'Glag',
    'Gothic'                            => 'Goth',
    'Grantha'                           => 'Gran',
    'Greek'                             => 'Grek',
    'Gujarati'                          => 'Gujr',
    'Gunjala_gondi'                     => 'Gong',
    'Gurmukhi'                          => 'Guru',
    'Han'                               => 'Hani',
    'Hangul'                            => 'Hang',
    'Hanifi_rohingya'                   => 'Rohg',
    'Hanunoo'                           => 'Hano',
    'Hatran'                            => 'Hatr',
    'Hebrew'                            => 'Hebr',
    'Hiragana'                          => 'Hira',
    'Imperial_aramaic'                  => 'Armi',
    'Inherited'                         => 'Zinh',
    'Inscriptional_pahlavi'             => 'Phli',
    'Inscriptional_parthian'            => 'Prti',
    'Javanese'                          => 'Java',
    'Kaithi'                            => 'Kthi',
    'Kannada'                           => 'Knda',
    'Katakana'                          => 'Kana',
    'Kawi'                              => 'Kawi',
    'Kayah_li'                          => 'Kali',
    'Kharoshthi'                        => 'Khar',
    'Khitan_small_script'               => 'Kits',
    'Khmer'                             => 'Khmr',
    'Khojki'                            => 'Khoj',
    'Khudawadi'                         => 'Sind',
    'Lao'                               => 'Laoo',
    'Latin'                             => 'Latn',
    'Lepcha'                            => 'Lepc',
    'Limbu'                             => 'Limb',
    'Linear_a'                          => 'Lina',
    'Linear_b'                          => 'Linb',
    'Lisu'                              => 'Lisu',
    'Lycian'                            => 'Lyci',
    'Lydian'                            => 'Lydi',
    'Mahajani'                          => 'Mahj',
    'Makasar'                           => 'Maka',
    'Malayalam'                         => 'Mlym',
    'Mandaic'                           => 'Mand',
    'Manichaean'                        => 'Mani',
    'Marchen'                           => 'Marc',
    'Masaram_gondi'                     => 'Gonm',
    'Medefaidrin'                       => 'Medf',
    'Meetei_mayek'                      => 'Mtei',
    'Mende_kikakui'                     => 'Mend',
    'Meroitic_cursive'                  => 'Merc',
    'Meroitic_hieroglyphs'              => 'Mero',
    'Miao'                              => 'Plrd',
    'Modi'                              => 'Modi',
    'Mongolian'                         => 'Mong',
    'Mro'                               => 'Mroo',
    'Multani'                           => 'Mult',
    'Myanmar'                           => 'Mymr',
    'Nabataean'                         => 'Nbat',
    'Nag_mundari'                       => 'Nagm',
    'Nandinagari'                       => 'Nand',
    'New_tai_lue'                       => 'Talu',
    'Newa'                              => 'Newa',
    'Nko'                               => 'Nkoo',
    'Nushu'                             => 'Nshu',
    'Nyiakeng_puachue_hmong'            => 'Hmnp',
    'Ogham'                             => 'Ogam',
    'Ol_chiki'                          => 'Olck',
    'Old_hungarian'                     => 'Hung',
    'Old_italic'                        => 'Ital',
    'Old_north_arabian'                 => 'Narb',
    'Old_permic'                        => 'Perm',
    'Old_persian'                       => 'Xpeo',
    'Old_sogdian'                       => 'Sogo',
    'Old_south_arabian'                 => 'Sarb',
    'Old_turkic'                        => 'Orkh',
    'Old_uyghur'                        => 'Ougr',
    'Oriya'                             => 'Orya',
    'Osage'                             => 'Osge',
    'Osmanya'                           => 'Osma',
    'Pahawh_hmong'                      => 'Hmng',
    'Palmyrene'                         => 'Palm',
    'Pau_cin_hau'                       => 'Pauc',
    'Phags_pa'                          => 'Phag',
    'Phoenician'                        => 'Phnx',
    'Psalter_pahlavi'                   => 'Phlp',
    'Rejang'                            => 'Rjng',
    'Runic'                             => 'Runr',
    'Samaritan'                         => 'Samr',
    'Saurashtra'                        => 'Saur',
    'Sharada'                           => 'Shrd',
    'Shavian'                           => 'Shaw',
    'Siddham'                           => 'Sidd',
    'SignWriting'                       => 'Sgnw',
    'Sinhala'                           => 'Sinh',
    'Sogdian'                           => 'Sogd',
    'Sora_sompeng'                      => 'Sora',
    'Soyombo'                           => 'Soyo',
    'Sundanese'                         => 'Sund',
    'Syloti_nagri'                      => 'Sylo',
    'Syriac'                            => 'Syrc',
    'Tagalog'                           => 'Tglg',
    'Tagbanwa'                          => 'Tagb',
    'Tai_le'                            => 'Tale',
    'Tai_tham'                          => 'Lana',
    'Tai_viet'                          => 'Tavt',
    'Takri'                             => 'Takr',
    'Tamil'                             => 'Taml',
    'Tangsa'                            => 'Tnsa',
    'Tangut'                            => 'Tang',
    'Telugu'                            => 'Telu',
    'Thaana'                            => 'Thaa',
    'Thai'                              => 'Thai',
    'Tibetan'                           => 'Tibt',
    'Tifinagh'                          => 'Tfng',
    'Tirhuta'                           => 'Tirh',
    'Toto'                              => 'Toto',
    'Ugaritic'                          => 'Ugar',
    'Unknown'                           => 'Zzzz',
    'Vai'                               => 'Vaii',
    'Vithkuqi'                          => 'Vith',
    'Wancho'                            => 'Wcho',
    'Warang_citi'                       => 'Wara',
    'Yezidi'                            => 'Yezi',
    'Yi'                                => 'Yiii',
    'Zanabazar_square'                  => 'Zanb',
);

1;
